<?php

namespace Drupal\brevo_webform_handler\Plugin\WebformHandler;

use Brevo\Client\Api\ContactsApi;
use Brevo\Client\Configuration;
use Brevo\Client\Model\CreateContact;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;

/**
 * Brevo Webform handler.
 *
 * @WebformHandler(
 *   id = "brevo_webform_handler",
 *   label = @Translation("Brevo Webform Handler"),
 *   category = @Translation("Transaction"),
 *   description = @Translation("Sends the submission data to Brevo Contacts API."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 *   submission = \Drupal\webform\Plugin\WebformHandlerInterface::SUBMISSION_OPTIONAL,
 * )
 */
class BrevoWebformHandler extends WebformHandlerBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'api_key' => '',
      'list_id' => '',
      'email' => '',
      'name' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    return [
      '#markup' => $this->t('No summary available'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $this->applyFormStateToConfiguration($form_state);

    // Define Ajax-callback.
    $ajax = [
      'callback' => [get_class($this), 'ajaxCallback'],
      'wrapper' => 'brevo-webform-handler--api-settings',
    ];

    // API settings.
    $form['api_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Brevo API settings'),
      '#attributes' => ['id' => 'brevo-webform-handler--api-settings'],
    ];

    $form['api_settings']['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Brevo API-key'),
      '#default_value' => $this->configuration['api_key'],
    ];


    $form['api_settings']['update_lists'] = [
      '#type' => 'submit',
      '#value' => $this->t('Update Brevo lists'),
      '#ajax' => $ajax,
      '#submit' => [[get_class($this), 'updateConfigSubmit']],
    ];

    $form['api_settings']['list_id'] = [
      '#type' => 'select',
      '#title' => $this->t('List'),
      '#empty_option' => $this->t('- Select a List -'),
      '#default_value' => $this->configuration['list_id'],
      '#options' => $this->getLists(),
      //'#ajax' => $ajax,
      '#description' => $this->t('Select the list you want to send this submission to.'),
    ];

    // Brevo fields if lists found.
    if (!empty($this->getLists())) {

      if (isset($this->configuration['list_id'])) {
        // Get webform elements as base for mapping.
        $elements = $this->webform->getElementsDecodedAndFlattened();
        foreach($elements as $key => $element) {
          if(!isset($element['#title']) || in_array($element['#type'], ['webform_actions'])) {
            unset($elements[$key]);
          }
        }
        $options = [];
        foreach($elements as $key => $element) {
          $options[$key] = $element['#title'];
        }

        // Email is the only required.
        $form['api_settings']['fields']['email'] = [
          '#type' => 'select',
          '#title' => $this->t('Email'),
          '#options' => $options,
          '#required' => TRUE,
          '#default_value' => $this->configuration['email'],
        ];

        // Get the merge fields for the extra field selections.
        $merge_fields = $this->getMergeFields($this->configuration['list_id']);


        foreach ($merge_fields as $brevo_key => $brevo_name) {
          $form['api_settings']['fields'][$brevo_key] = [
            '#type' => 'select',
            '#title' => $brevo_name,
            '#options' => ['' => ''] + $options,
            '#default_value' => $this->configuration[$brevo_key],
          ];
        }
      }
    }

    return $this->setSettingsParents($form);
  }

  /**
   * Submit callback for the refresh button.
   */
  public function updateConfigSubmit(array $form, FormStateInterface $form_state) {
    $form_state->setRebuild();
  }


  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['api_key'] = $form_state->getValue('api_key');
    $this->configuration['list_id'] = $form_state->getValue('list_id');
    $this->configuration['email'] = $form_state->getValue('email');

    if ($form_state->getValue('list_id')) {
      // Save configuration for extra MERGE fields.
      $merge_fields = $this->getMergeFields($this->configuration['list_id']);

      foreach ($merge_fields as $brevo_key => $brevo_name) {
        $this->configuration[$brevo_key] = $form_state->getValues()['api_settings']['fields'][$brevo_key];
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state,  WebformSubmissionInterface $webform_submission) {
    // Get the submission data.
    $values = $webform_submission->getData();
    $config = Configuration::getDefaultConfiguration()->setApiKey('api-key', $this->configuration['api_key']);

    $client = new ContactsApi(
      new Client(),
      $config
    );

    // Post to Brevo ContactsApi.
    $contact = new CreateContact();
    $contact['email'] = $values[$this->configuration['email']];

    // Get List ID's.
    $list_id = $this->configuration['list_id'];
    $contact['listIds'] = [intval($list_id)];
    $contact['updateEnabled'] = TRUE;

    // Add merge fields.
    $merge_fields = $this->getMergeFields($list_id);
    $attributes_data = [];

    foreach ($merge_fields as $brevo_key => $brevo_name) {
      if (isset($this->configuration[$brevo_key])
        && isset($values[$this->configuration[$brevo_key]])
        && !is_null($values[$this->configuration[$brevo_key]])) {
        $attributes_data[$brevo_key] = $values[$this->configuration[$brevo_key]];
      }
    }

    $contact['attributes'] = (object)$attributes_data;

    try {
      $response = $client->createContact($contact);

    }
    catch (RequestException $request_exception) {
    }

  }

  /**
   * Get Brevo lists.
   */
  protected function getLists() {
    $lists = [];

    if (isset($this->configuration['api_key']) && $this->configuration['api_key'] != '') {
      $config = Configuration::getDefaultConfiguration()->setApiKey('api-key', $this->configuration['api_key']);

      $client = new ContactsApi(
        new Client(),
        $config
      );

      $lists_array = [];
      $count_lists = 50;

      while ($count_lists == 50) {
        try {
          // 50 is the max. limit.
          $response = $client->getLists(50, count($lists_array));
          $results = $response->getLists();

          // Append to lists array.
          $lists_array += $results;

          $count_lists = count($results);

        }
        catch (RequestException $request_exception) {
        }
      }

      if (isset($lists_array) && !empty($lists_array)) {
        foreach ($lists_array as $list) {

          $lists[$list['id']] = $list['name'];
        }
      }
    }

    return $lists;
  }

  /**
   * Get the merge fields for a specific list.
   *
   * @param $list_id
   */
  protected function getMergeFields($list_id) {
    $merge_fields = [];

    if (isset($this->configuration['api_key']) && $this->configuration['api_key'] != '') {
      $config = Configuration::getDefaultConfiguration()->setApiKey('api-key', $this->configuration['api_key']);

      $client = new ContactsApi(
        new Client(),
        $config
      );
    }


    try {
      $response = $client->getAttributes();
    }
    catch (RequestException $request_exception) {
    }

    if (isset($response)) {
      foreach ($response['attributes'] as $attribute) {
        if (isset($attribute['category']) && $attribute['category'] == 'normal') {
          $merge_fields[$attribute['name']] = $attribute['name'];
        }
      }
    }

    return $merge_fields;
  }

  /**
   * Ajax callback.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   An associative array containing entity reference details element.
   */
  public function ajaxCallback(array $form, FormStateInterface $form_state) {
    return NestedArray::getValue($form, ['settings', 'api_settings']);
  }

}
